package fr.faraphel.m1_pe_kafka.kafka

/**
 * The consumer
 */

import org.apache.kafka.clients.consumer.*
import org.apache.kafka.common.serialization.DoubleDeserializer
import org.apache.kafka.common.serialization.StringDeserializer

import java.time.Duration
import java.util.Collections
import java.util.Properties


/**
 * The consumer
 */
class Consumer(
    private val topic: String
) {
    private val properties: Properties = Properties().apply {
        this[ConsumerConfig.GROUP_ID_CONFIG] = "consumer-group"
        this[ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG] = StringDeserializer::class.java.name
        this[ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG] = DoubleDeserializer::class.java.name
        this[ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG] = System.getenv("KAFKA_BOOTSTRAP_SERVERS")
            ?: throw IllegalArgumentException("Missing environment variable: KAFKA_BOOTSTRAP_SERVERS")
    }

    private val consumer = KafkaConsumer<String, Double>(this.properties).apply {
        this.subscribe(Collections.singletonList(topic))
    }

    fun consume() {
        val messages: ConsumerRecords<String, Double> = this.consumer.poll(Duration.ofMillis(500));

        messages.forEach { message ->
            println(">" + message.value())
        }
    }

    fun consumeForever() {
        while (true) this.consume()
    }
}
