package fr.faraphel.m1_pe_kafka

import fr.faraphel.m1_pe_kafka.kafka.Consumer
import fr.faraphel.m1_pe_kafka.kafka.Converter
import fr.faraphel.m1_pe_kafka.kafka.TemperatureProducer
import io.quarkus.runtime.Quarkus
import io.quarkus.runtime.QuarkusApplication
import io.quarkus.runtime.annotations.QuarkusMain
import okhttp3.internal.wait
import org.apache.kafka.clients.admin.AdminClient
import org.apache.kafka.clients.admin.AdminClientConfig
import org.apache.kafka.clients.admin.NewTopic
import java.util.*


/**
 * The entrypoint of the program
 */
@QuarkusMain
class Main : QuarkusApplication {
    override fun run(vararg args: String?): Int {
        println("starting...")

        val adminConfig = Properties().apply {
            this[AdminClientConfig.CLIENT_ID_CONFIG] = "main"
            this[AdminClientConfig.BOOTSTRAP_SERVERS_CONFIG] = System.getenv("KAFKA_BOOTSTRAP_SERVERS")
                ?: throw IllegalArgumentException("Missing environment variable: KAFKA_BOOTSTRAP_SERVERS")
        }
        val admin = AdminClient.create(adminConfig)
        admin.createTopics(mutableListOf(
            NewTopic("temperature-celsius", 1, 1),
            NewTopic("temperature-fahrenheit", 1, 1),
        )).all().get()

        val producer = TemperatureProducer(49.9, 2.3, "temperature-celsius")
        val converter = Converter("temperature-celsius", "temperature-fahrenheit")
        val consumer = Consumer("temperature-fahrenheit")

        val threadProducer = Thread { producer.produceForever() }
        val threadConsumer = Thread { consumer.consumeForever() }

        threadConsumer.start()
        converter.start()
        threadProducer.start()

        threadConsumer.join()
        threadProducer.join()
        converter.stop()

        return 0
    }
}

/**
 * The main function
 * Call the entrypoint directly. Used to run the program directly with Kotlin
 */
fun main(args: Array<String>) {
    Quarkus.run(Main::class.java, *args)
}
